% Solve VP to recover wafer map from a small number of measurements
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SampleData(i,1):  measured chip performances
% DataFlag(i,j):    wafer data flag
%                   1 if (i,j) is a chip
%                   0 otherwise
% SampleX(i,1):     coordinates of measurements (X-axis)
% SampleY(i,1):     coordinates of measurements (Y-axis)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PerfEst(i,j):     recovered wafer map by VP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function PerfEst = solve_vp_accu(SampleData,DataFlag,SampleX,SampleY)

NumExp = length(SampleX);
XMax = size(DataFlag,1);
YMax = size(DataFlag,2);
NumF = XMax*YMax;

% normalize data
cur_mean = mean(SampleData);
cur_std = std(SampleData);
SampleData_nor = (SampleData-cur_mean)/cur_std;

% formulate L1 problem
EqnA = buildEqnA(DataFlag,SampleX,SampleY);
EqnB = SampleData_nor;

% normalize columns of linear equation
MeanA = mean(EqnA,1);
DCIndex = 1; MeanA(DCIndex) = 0;
EqnA = EqnA - repmat(MeanA,NumExp,1);
ScaleA = sqrt(sum(EqnA.^2,1))/sqrt(NumExp);

% eliminate numerical issues in scaling
MeanScale = mean(ScaleA);
for i = 1:NumF
    if ScaleA(i) < 1e-2 * MeanScale
        ScaleA(i) = MeanScale;
    end
end

EqnA = EqnA ./ repmat(ScaleA,NumExp,1);

% solve linear programming
DataF = l1eq_pd(EqnA\EqnB, EqnA, [], EqnB, 1e-3);

% obtain DCT coefficients from LP solution
DataF = DataF ./ ScaleA';
DataF(DCIndex) = DataF(DCIndex) - MeanA*DataF/ScaleA(DCIndex);
PerfDCT = reshape(DataF,XMax,YMax);

% recover performance
PerfEst = idct2(PerfDCT);
PerfEst = (PerfEst*cur_std) + cur_mean;
PerfEst(DataFlag==0) = NaN;

% set known measurements to exact value
for i=1:NumExp
    PerfEst(SampleX(i),SampleY(i)) = SampleData(i);
end
