% Recover wafer map from partial measurements considering measurement noise
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SampleData(i,1):  measured chip performances
% DataFlag(i,j):    wafer data flag
%                   1 if (i,j) is a chip
%                   0 otherwise
% SampleX(i,1):     coordinates of measurements (X-axis)
% SampleY(i,1):     coordinates of measurements (Y-axis)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PerfEst(i,j):     recovered wafer map by VP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function PerfEst = solve_vp_noise(SampleData,DataFlag,SampleX,SampleY)

NumExp = length(SampleX);
XMax = size(DataFlag,1);
YMax = size(DataFlag,2);

% normalize data
cur_mean = mean(SampleData);
cur_std = std(SampleData);
SampleData_nor = (SampleData-cur_mean)/cur_std;

% formulate L1 problem
EqnA = buildEqnA(DataFlag,SampleX,SampleY);
EqnB = SampleData_nor;

% perform cross-validation to determine lambda
cv_folds = 10;
base_num = floor(NumExp / cv_folds);
mod_num = mod(NumExp,cv_folds);

% determine the indices for cross-validation folds
cv_index = [];
for j = 1:mod_num
    cv_index = [cv_index;ones(base_num+1,1)*j];
end
for j = mod_num+1:cv_folds
    cv_index = [cv_index;ones(base_num,1)*j];
end

% list of lambda values
lambda = [1e-3 1e-2 1e-1 3e-1 6e-1 1 3 10 30 100];

% cross-validation
for i = 1:cv_folds
    cur_train_index = find(cv_index ~= i);
    EqnA_train = EqnA(cur_train_index,:);
    EqnB_train = EqnB(cur_train_index);
    
    cur_test_index = find(cv_index == i);
    EqnB_test = EqnB(cur_test_index);
    EqnB_test_pred = EqnB_test*0;
    SampleX_test = SampleX(cur_test_index);
    SampleY_test = SampleY(cur_test_index);
    
    % normalize data  
    fold_mean = mean(EqnB_train);
    fold_std = std(EqnB_train);
    EqnB_train = (EqnB_train-fold_mean)/fold_std;
    for j = 1:length(lambda)
        %PerfDCT = get_fastL1_nor(PerfData_temp,SampleX_train,SampleY_train,lambda(j));
        PerfDCT = solve_l1_ineq(EqnA_train,EqnB_train,lambda(j));
        PerfDCT = reshape(PerfDCT,XMax,YMax);
        PerfEst = idct2(PerfDCT);
        PerfEst = (PerfEst * fold_std) + fold_mean;
        for k = 1:length(cur_test_index)
            EqnB_test_pred(k) = PerfEst(SampleX_test(k),SampleY_test(k));
        end
        error(i,j) = norm(EqnB_test_pred-EqnB_test)/norm(EqnB_test);
    end
end
error_lambda = sqrt(mean(error.^2,1));
[temp ind] = min(error_lambda);
lambda_ind = ind;


% solve actual vp problem
PerfDCT = solve_l1_ineq(EqnA,EqnB,lambda(ind));
PerfDCT = reshape(PerfDCT,XMax,YMax);

% recover performance
PerfEst = idct2(PerfDCT);
PerfEst = (PerfEst*cur_std) + cur_mean;
PerfEst(DataFlag==0) = NaN;
